/****************************************************************************
 * Copyright Torquil Gault 2005+
 *
#       This program is free software; you can redistribute it and/or
#       modify it under the terms of the GNU General Public License
#       as published by the Free Software Foundation; either version 2
#       of the License, or (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
###############################################################################
****************************************************************************/


#define MAX_WEAPONS				10
#define MAX_TEMPPENALTIES		8
#define MAX_BAB					3
#define MAX_ABILITYSTATS		6
#define MAX_CLASSES				4
#define MAX_SPELLDCS				9
#define MAX_D20ROLLS				9
#define MAX_SKILLS				9
#define MAX_GROUPS				10
#define MAX_MONSTERS				42
#define MAX_PLAYERS				10
#define MAX_MODIFIERS			20
#define MAX_TRIGGERS				10
#define MAX_ATTACKS				10
#define MAX_AUTODAMAGE			10
#define MAX_WEAPONATTACKS		4

#define MAX_QKMONSTERS			663

#define MAX_MEMBERS				(MAX_MONSTERS+MAX_PLAYERS)

#define MAX_STDWEAPONS			66

#define AC_NORMAL					0
#define AC_TOUCH					1
#define AC_FLATFOOTED			2

#define SAVE_FORTITUDE			0
#define SAVE_REFLEX				1
#define SAVE_WISDOM				2

#define HP_MAX						0
#define HP_CUR						1
#define HP_NL						2
#define HP_TMP						3

#define ABILITY_STR				0
#define ABILITY_DEX				1
#define ABILITY_CON				2
#define ABILITY_INT				3
#define ABILITY_WIS				4
#define ABILITY_CHA				5

#define MAX							0
#define TMP							1

#define SKILL_SEARCH				0
#define SKILL_SOH					1
#define SKILL_SPOT				2
#define SKILL_OPENLOCKS			3
#define SKILL_MOVESILENTLY		4
#define SKILL_LISTEN				5
#define SKILL_HIDE				6
#define SKILL_CONCENTRATION	7

#define BAB_MELEE					0
#define BAB_RANGED				1
#define BAB_GRAPPLE				2

#define TYPE						0
#define VALUE						1

#define MOD_BAB					0
#define MOD_AC						1
#define MOD_SAVE					2
#define MOD_DAMAGE				3
#define MOD_INIT					4
#define MOD_HP						5
#define MOD_SKILL					6

#define AC_SECTION_DEX			0
#define AC_SECTION_SIZE			1
#define AC_SECTION_DEFLECTION	2
#define AC_SECTION_NATURAL		3
#define AC_SECTION_ARMOR		4
#define AC_SECTION_SHIELD		5

	// these must match the Spell Options sequence
#define SO_SPELLRESISTANCE		1

#define ID_EDIT					0		// a max of 15 ie. 0-14 lists
#define ID_INIT 					1
#define ID_DEFD					2
#define ID_WEAP					3
#define ID_QMON					4
#define ID_ADJ1					5
#define ID_ADAM					6
#define ID_TEMP 					7
#define ID_WSCH 					8
#define ID_DISA 					9
#define ID_MSKILLCHK 			10


typedef	unsigned char uchar;

/*********************************************************************/
typedef struct ADMPweapon {
	char	c_available;			/* is this available to be used */
	char	c_description[30];	/* weapon description */
	char	i_attackBonus[4];		/* attack bonuses */
	int	i_noAttacks;			/* no. attacks with this weapon */
	int	i_doneAttacks;			/* no. attacks done with this weapon */
	char	c_damage[10];			/* ie. 3d4, +3 wis etc */
	char	c_critical[10];		/* crit range ie. 18-20,*3 */
	int	i_range;					/* base range of the weapon */
	int	i_reach;					/* weapon reach */
	int	i_magicalBonus;		/* used in Dam. Reduction calcs */
	char	c_size;					/* weapon size */
	char	c_type;					/* weapon type = Slash/Puncturing etc */
	char	c_attackType;			/* melee or ranged etc */
	char	c_message[30];			/* a message to display about this weapon or attack */
} ADMPweapon;

/*********************************************************************/
typedef struct ADMPflags {
	char	f_isStabilised:1;
	char	f_disabled:1;
	char	f_ignoreCriticals:1;
	char	f_autoStablise:1;
	char	f_flag5:1;
	char	f_flag6:1;
	char	f_flag7:1;
	char	f_flag8:1;
	char	f_flag9:1;
	char	f_flag10:1;
	char	f_flag11:1;
	char	f_flag12:1;
	char	f_flag13:1;
	char	f_flag14:1;
	char	f_flag15:1;
	char	f_flag16:1;
} ADMPflags;

typedef struct ADMPflags2 {
	char	f_flag1:1;
	char	f_flag2:1;
	char	f_flag3:1;
	char	f_flag4:1;
	char	f_flag5:1;
	char	f_flag6:1;
	char	f_flag7:1;
	char	f_flag8:1;
} ADMPflags2;

typedef struct ADMPflags3 {
	char	f_debug:1;
	char	f_rollAllRolls:1;
	char	f_rollInitRolls:1;
	char	f_logfile:1;
	char	f_rollInitEveryRnd:1;
	char	f_removeMonstersInit:1;
	char	f_oneMissTwentyHits:1;
	char	f_flag8:1;
	char	f_flag9:1;
	char	f_flag10:1;
	char	f_flag11:1;
	char	f_flag12:1;
	char	f_flag13:1;
	char	f_flag14:1;
	char	f_flag15:1;
	char	f_flag16:1;
} ADMPflags3;

typedef struct ADMPspell {
	char	i_maxLevel0:4;
	char	i_curLevel0:4;
	char	i_maxLevel1:4;
	char	i_curLevel1:4;
	char	i_maxLevel2:4;
	char	i_curLevel2:4;
	char	i_maxLevel3:4;
	char	i_curLevel3:4;
	char	i_maxLevel4:4;
	char	i_curLevel4:4;
	char	i_maxLevel5:4;
	char	i_curLevel5:4;
	char	i_maxLevel6:4;
	char	i_curLevel6:4;
	char	i_maxLevel7:4;
	char	i_curLevel7:4;
	char	i_maxLevel8:4;
	char	i_curLevel8:4;
	char	i_maxLevel9:4;
	char	i_curLevel9:4;
} ADMPspell;

/*********************************************************************/
typedef struct ADMPplayer {
	char	c_name[30];
	int	i_classes[MAX_CLASSES];
	int	i_levels[MAX_CLASSES];
	int	i_race;
	int	i_alignment;
	int	i_size;
	int	i_castSpells;

	float	f_challengeRating;
	int	i_d20Rolls[MAX_D20ROLLS];
	int	i_skills[MAX_SKILLS];

	int	i_speed;
	int	i_initiative[2];
	int	i_armorClass[4];
	int	i_armorClassSplit[8];						/* DEX, Size, Deflection etc */
	char	i_bab[MAX_BAB][4];							/* base attack bonuses */

	int	i_reach;											/* standard reach of the player */
	int	i_space;											/* standard space taken of the player */

	int	i_inGroup;

	int	i_noAttacks;
	int	i_doneAttacks;
	short int	i_whomAttacked[MAX_ATTACKS+2][2];/* the ID of whom they attacked last time & weapon used */

	int	i_xp[2];
	int	i_abilityStats[2][MAX_ABILITYSTATS];	/* abilitiy stats Curr & Temp */

	int	i_spellFailure;								/* spell failure percentage */
	int	i_spellDC[MAX_SPELLDCS];
	int	i_savingThrows[4];
	int	i_spellResistance;

	int	i_hp[4];											/* Max, Current, Non-lethal, Temp */
	int	i_bleeding;										/* per round amount, can be used for regeneration */
	int	i_disabledAt;									/* usually at 0 */
	int	i_unconciousAt;								/* usually at -1 */
	int	i_inactive;										/* no rounds inactive */
	int	i_stun;											/* no rounds stunned */
	short int	i_damageReduction[4];				/* ie. 8/+2 */
	ADMPflags	flags;
	ADMPspell	spellsAvailable;						/* spells available at each level */

	int	i_dummy0;

	int	i_tempPenalties[3][MAX_TEMPPENALTIES];	/* Type, Value, Rounds */

	char	c_hitDice[20];									/* Hit Dice */
	uchar	c_environment;									/* where it lives - monsters only */
	uchar	i_dummy[9];									/* dummy space to use up */

	ADMPweapon	w_weapons[MAX_ATTACKS+2];
} ADMPplayer;

/*********************************************************************/
typedef struct ADMPmodifiers {
	char	c_message[40];					// the message to display
	int	i_idFollows;					// which ID does it follow to finish on
	int	i_noRounds;						/* how many rounds until it lasts */
	int	i_whom[MAX_MEMBERS];			/* whom it effects */
	int	i_data[5][3];					/* the value, type, effects-type */
} ADMPmodifiers;

/*********************************************************************/
typedef struct ADMPtriggers {
	char	c_message[60];		/* whats the message */
	int	i_noRounds;			/* how many rounds until it happens */
	int	i_display;			/* should it be displayed every round */
} ADMPtriggers;

/*********************************************************************/
typedef struct ADMPgroup {
	char	c_description[30];
	int	i_initiative;
	int	i_members[MAX_MEMBERS+2];
} ADMPgroup;

/*********************************************************************/
typedef struct ADMPdamage {
	char	c_description[32];	// what's it called
	int	i_initiatorID;			// who did the auto-damage
	int	i_whom[MAX_MEMBERS+2];		// who 'gets' it
	char	c_damage[32];			// what damage to do .. either a dice roll or a series of numbers
	int	i_noRounds;				// how may rounds to last
	int	i_doneRounds;			// how may rounds done
	int	i_type;					// what type of damage to do
	ADMPflags2	flags;			// any flags - like was it done this round, follow initiator's init etc
} ADMPdamage;

/*********************************************************************/
typedef struct ADMPsystem {
	int			i_roundCount;
	int			i_whenBleed;							/* when to bleed, at INIT or end of rnd */
	int			i_xpAllocate;							/* how to do XP calculations */
	int			i_doGroupInitiatives;				/* do groups as ONE initiative roll */

	int			i_attackGroup;							/* do all attacks at once or separate them */
	int			i_attackSeparation[MAX_ATTACKS]; /* base INIT modifiers for each attack */

	int			i_editID;
	int			i_actionInitID;
	int			i_actionRecvID;
	int			i_actionSelected;
	int			i_actionWeaponSelected;

	int			i_roundsSave;							/* every X rounds save the player/monster data */

	char			c_partyFilename[44];
	char			c_monstFilename[44];
	char			c_mapFilename[2];

	int			i_dummy1;									/* show debugging messages or not */

	int			i_idList[15][MAX_MEMBERS+2];

	ADMPflags3	flags;
	short int	i_dummy2;
	int			i_dummy3;

	int			i_dummy4;

	ADMPmodifiers	system_mods[MAX_MODIFIERS];
	ADMPmodifiers	action_mods[MAX_MODIFIERS];
	ADMPtriggers	action_msgs[MAX_TRIGGERS];
	ADMPgroup		groups[MAX_GROUPS];
	ADMPdamage		auto_damage[MAX_AUTODAMAGE];

} ADMPsystem;

/*********************************************************************/
typedef struct ADMPcombatmods {
	char *c_message;
	int   i_modifications[2][2];
} ADMPcombatmods;

/*********************************************************************/
typedef struct ADMPadjustments {
	char	*title;
	int	i_type;
} ADMPadjustments;
